import os
import sys

sys.path.append(os.getcwd() + "/src")

import pyglet
from pyglet.gl import *
from pyglet.window import key
from pyglet.window import mouse

from creatures.common import constants
from creatures.common.tileset import TilesetFactory
from creatures.editor.map_renderer import EditorMapRendererFactory
from creatures.common.map_system import MapFactory
from tile_chooser_interface import TileChooserInterface


WINDOW_WIDTH = 800
WINDOW_HEIGHT = 600

# create the window, but keep it offscreen until we are done with setup
window = pyglet.window.Window(WINDOW_WIDTH, WINDOW_HEIGHT, visible=False, caption="Pick a Tile")

# centre the window on whichever screen it is currently on (in case of multiple monitors)
window.set_location(window.screen.width/2 - window.width/2, window.screen.height/2 - window.height/2)

# handle the window resize event
@window.event
def on_resize(width, height):
    glViewport(0, 0, width, height)

    glMatrixMode(GL_PROJECTION)
    glLoadIdentity()
    # setup a fixed dimension orthographic projection
    # this avoids gameplay problems if the window resolution is changed
    glOrtho(0, WINDOW_WIDTH, 0, WINDOW_HEIGHT, -1, 1)
    glMatrixMode(GL_MODELVIEW)
    glLoadIdentity()

    # tell pyglet that we have handled the event, prevents the default handler from running
    return pyglet.event.EVENT_HANDLED

# setup a key handler to track keyboard input
keymap = pyglet.window.key.KeyStateHandler()
window.push_handlers(keymap)


# clear the window and draw the scene
@window.event
def on_draw():
    window.clear()
    user_interface.draw()

@window.event
def on_mouse_motion(x, y, dx, dy):
    user_interface.set_mouse_position(x, y)

@window.event
def on_mouse_release(x, y, button, modifiers):
    if button == mouse.LEFT:
        user_interface.set_mouse_clicked()


# clear and flip the window, otherwise we see junk in the buffer before the first frame
window.clear()
window.flip()

# make the window visible at last
window.set_visible(True)

# update callback
key_handler = key.KeyStateHandler()
window.push_handlers(key_handler)

map_name = sys.argv[1]

window_center = WINDOW_WIDTH / 2, WINDOW_HEIGHT / 2

# load tile sets and create game objects
tileset_factory = TilesetFactory(constants.IMAGE_DIR, constants.TILESETS)
map_factory = MapFactory(constants.MAP_DIR)
map_data = map_factory.load_map(map_name)
map_renderer_factory = EditorMapRendererFactory(tileset_factory)
map_renderer = map_renderer_factory.create_renderer(map_data, window_center)
user_interface = TileChooserInterface(key_handler, WINDOW_HEIGHT, WINDOW_WIDTH, \
                                          map_renderer)
user_interface.draw()

# schedule the update function, 30 times per second
pyglet.clock.schedule_interval(user_interface, 1.0/30.0)


# finally, run the application
pyglet.app.run()
