"""
High level editor GUI.  Allows switching between the sub-GUIs for editing maps
and choosing tiles.
"""


import sys

from pyglet import text
from pyglet.window import key

from creatures.editor import dialog_utilities
from creatures.editor import drawing_utilities
from creatures.common import geometry_utilities
from creatures.editor.widgets import TextButton


class EditorUserInterface(object):
    def __init__(self, key_handler, window_height, window_width, map_editor_interface, \
                     tile_type_chooser_interface):
        """
        map_name -- String.  If given, that map is loaded.  Otherwise, a blank map is
        created.
        """
        self.key_handler = key_handler
        self.window_height = window_height
        self.window_width = window_width

        # None means the mouse has not been initialized
        self.mouse_x = None
        self.mouse_y = None

        self.mouse_clicked = False

        self.map_editor_button = TextButton(x = 10, y = self.window_height - 10 - 50, \
                                 width = 150, height = 50, text = "Map", \
                                 background = drawing_utilities.BLACK, \
                                 border = drawing_utilities.RED, \
                                 text_color = drawing_utilities.WHITE)

        self.tile_type_chooser_button = TextButton(x = 10 + 150 + 10,
                                          y = self.window_height - 10 - 50,
                                          width = 150,
                                          height = 50,
                                          text = "Tiles",
                                          background = drawing_utilities.BLACK,
                                          border = drawing_utilities.RED,
                                          text_color = drawing_utilities.WHITE)

        self.map_editor_interface = map_editor_interface

        self.tile_type_chooser_interface = tile_type_chooser_interface

        self.current_interface = self.map_editor_interface

    def __call__(self, dt):
        """
        Adapter for using UserInterface as update(dt) function. Calls
        self.update(dt)
        """
        self.update(dt)

    def update(self, dt):
        # quit program
        if self.key_handler[key.Q]:
            sys.exit()

        if self.mouse_clicked:
            if self.map_editor_button.is_clicked(self.mouse_x, self.mouse_y):
                self.current_interface = self.map_editor_interface
                self.map_editor_interface.set_current_tile_type(self.tile_type_chooser_interface.current_tile_type)
            elif self.tile_type_chooser_button.is_clicked(self.mouse_x, self.mouse_y):
                self.current_interface = self.tile_type_chooser_interface
            else:
                self.current_interface.set_mouse_clicked()

            self.draw()
            self.clear_mouse_clicked()

        self.current_interface.update(dt)


    def draw_buttons(self):
        self.map_editor_button.draw()
        self.tile_type_chooser_button.draw()

    def set_mouse_position(self, x, y):
        self.mouse_x = x
        self.mouse_y = y
        self.current_interface.set_mouse_position(x, y)

    def set_mouse_clicked(self):
        self.mouse_clicked = True

    def clear_mouse_clicked(self):
        self.mouse_clicked = False

    def draw(self):
        # Order is important here
        self.current_interface.draw()
        self.draw_buttons()
