"""
Contains functions for detecting collisions.
"""

from creatures.common import geometry_utilities

class CollisionDetector(object):
    def __init__(self, map_data, map_renderer):
        self.map_data = map_data
        self.map_renderer = map_renderer

    def on_barrier(self, x, y):
        """
        Return True if the coordinates fall on a tile marked as a barrier,
        otherwise return False. Returns False for coordinates that fall
        outside the map.
        """
        if self.map_renderer.outside_map(x, y):
            return False
        tile_coord = self.map_renderer.coordinate_to_tile(x, y)
        return self.map_data.grid[tile_coord].barrier

    def teleport(self, sprite):
        """
        Return tuple of True, map name, and destination if the sprite overlaps with a tile marked
        as a teleport, otherwise return (False, None, None).
        """
        sprite_corners = geometry_utilities.four_corners(sprite)
        for corner_x, corner_y in sprite_corners:
            if self.map_renderer.outside_map(corner_x, corner_y):
                continue
            tile_coord = self.map_renderer.coordinate_to_tile(corner_x, corner_y)
            if self.map_data.grid[tile_coord].teleport:
                return True, \
                    self.map_data.grid[tile_coord].teleport_destination, \
                    self.map_data.grid[tile_coord].teleport_land
        return False, None, None

    def collision(self, sprite):
        """
        Return True if there is a collision. Otherwise, returns False.
        """
        sprite_corners = geometry_utilities.four_corners(sprite)
        for corner_x, corner_y in sprite_corners:
            if self.map_renderer.outside_map(corner_x, corner_y) or \
                    self.on_barrier(corner_x, corner_y):
                return True
        return False
