# mostly identical to Paddle, apart from size, offsets and velocity
class Asteroid(pyglet.sprite.Sprite):
        def __init__(self):
                image = pyglet.image.load("astroid.png")
                image.anchor_x, image.anchor_y = 10, 10

                pyglet.sprite.Sprite.__init__(self, image, batch=batch)

                # total velocity constant
                self.velocity = 100.0

                # reset ourselves
                self.reset()
                                
        def check_boundaries(self):
                if self.x >= 800:
                        self.x = 10
                if self.y >= 600:
                        self.y = 10
                if self.x <= 0:
                        self.x = 790
                if self.y <= 0:
                        self.y = 590




                
        def update(self, dt):
                self.x += self.velocity_x * dt
                self.y += self.velocity_y * dt
                
                self.check_boundaries()
                
        def reset(self):
                # place ourselves in the centre of the playing field
                self.x = random.randint(0, 800)
                self.y = random.randint(0, 600)
                
                self.angle = random.randint(0, 359)
                radians = math.radians(self.angle)
                self.velocity_x = math.cos(radians) * self.velocity
                self.velocity_y = math.sin(radians) * self.velocity



class Ship(pyglet.sprite.Sprite):
        def __init__(self):
                image = pyglet.image.load("crappy spaceship.png")
                image.anchor_x, image.anchor_y = 10, 10

                pyglet.sprite.Sprite.__init__(self, image, batch=batch)
                # reset ourselves
                self.reset()

                self.key_handler = key.KeyStateHandler()
                self.event_handlers = [self, self.key_handler]
                
                self.rotation = -90.0

                self.thrust = 300.0
                self.rotate_speed = 200.0

                self.velocity_x = 0.0
                self.velocity_y = 0.0


                
        def update(self, dt):
                if self.key_handler[key.LEFT]:
                        self.rotation -= self.rotate_speed * dt
                if self.key_handler[key.RIGHT]:
                        self.rotation += self.rotate_speed * dt
                
                if self.key_handler[key.UP]:
                        angle_radians = -math.radians(self.rotation)
                        force_x = math.cos(angle_radians) * self.thrust * dt
                        force_y = math.sin(angle_radians) * self.thrust * dt
                        self.velocity_x += force_x
                        self.velocity_y += force_y

                if self.key_handler[key.DOWN]:
                        angle_radians = -math.radians(self.rotation)
                        force_x = -1.0 * math.cos(angle_radians) * self.thrust * dt
                        force_y = -1.0 * math.sin(angle_radians) * self.thrust * dt
                        self.velocity_x += force_x
                        self.velocity_y += force_y
                self.x += self.velocity_x * dt
                self.y += self.velocity_y * dt
                self.check_boundaries()
                self.draw()



        def check_boundaries(self):
                if self.x >= 800:
                        self.x = 10
                if self.y >= 600:
                        self.y = 10
                if self.x <= 0:
                        self.x = 790
                if self.y <= 0:
                        self.y = 590

        def reset(self):
                # place ourselves in the centre of the playing field
                self.x = 20
                self.y = 20

class GameState:
        def __init__(self):
                self.asteroids = []
                self.ship = None
                self.state = WAITING

        def start_game(self):
                for i in xrange(10):
                        asteroid = Asteroid()
                        self.asteroids.append(asteroid)
                self.ship = Ship()
                for handler in self.ship.event_handlers:
                        window.push_handlers(handler)

        def update(self, dt):
                if self.state == WAITING:
                        if keymap[pyglet.window.key.SPACE]:
                                self.state = PLAYING
                                self.start_game()
                elif self.state == PLAYING:
                        self.ship.update(dt)
                        for asteroid in self.asteroids:
                                asteroid.update(dt)
                                if self.detect_collision(self.ship, asteroid):
                                        print "Collision"
                                        self.state = STOPPED
                                        pyglet.text.Label("YOU LOSE!", font_name = "Arial", font_size = 24, x = 400, y = 575, anchor_x = "center", anchor_y = "center", batch=batch)
                elif self.state == STOPPED:
                        if keymap[pyglet.window.key.SPACE]:
                                pyglet.app.exit()



        def pt_in_box(self, pt, bottomleft, upperright):
                # pt is above and to the right of bottom left
                # pt is below and to the left and upper right
                if pt[0] >= bottomleft[0] and pt[1] >= bottomleft[1] \
                        and pt[0] <= upperright[0] and pt[1] <= upperright[1]:
                                return True
                return False

        def detect_collision(self, sprite1, sprite2):
                sprite2_bottomleft = sprite2.position
                sprite2_upperright = (sprite2.x + sprite2.width, \
                                        sprite2.y + sprite2.height)
                sprite1_bottomleft = sprite1.position
                sprite1_upperright = (sprite1.x + sprite1.width, \
                                        sprite1.y + sprite1.height)
                sprite1_bottomright = (sprite1.x + sprite1.width, \
                                        sprite1.y)
                sprite1_upperleft = (sprite1.x, \
                                        sprite1.y + sprite1.height)
                if self.pt_in_box(sprite1_bottomleft, sprite2_bottomleft, \
                        sprite2_upperright):
                        return True
                if self.pt_in_box(sprite1_bottomright, sprite2_bottomleft, \
                        sprite2_upperright):
                        return True
                if self.pt_in_box(sprite1_upperleft, sprite2_bottomleft, \
                        sprite2_upperright):
                        return True
                if self.pt_in_box(sprite1_upperright, sprite2_bottomleft, \
                        sprite2_upperright):
                        return True
                return False
